import 'package:flutter/material.dart';
import 'package:vone_ipfs/utils/ipfs_utils.dart';
import 'package:vone_ipfs/widget/multiple_image_view.dart';
import 'package:vone_ipfs/widget/single_image_view.dart';
import 'package:vone_ipfs/widget/vone_image.dart';

class ImageUploadPage extends StatefulWidget {
  ///图片的宽度
  final double imageWidth;

  ///图片高度
  final double imageHeight;

  ///已有的图片列表
  final List<String> images;

  ///图片形状
  final ImageShape shape;

  ///上传模式 默认多文件上传
  final UploadMode uploadMode;

  ///上传成功的回调
  final ValueChanged<List<String>>? valueChangedCallback;

  ///添加图片的按钮
  final Widget? addImageWidget;

  /// 添加图片按钮的位置
  final UploadBtnPosition btnPosition;
  final int itemCount;

  ImageUploadPage(
      {Key? key,
      this.imageWidth = 110,
      this.imageHeight = 110,
      required this.images,
      this.uploadMode = UploadMode.MULTIPLE,
      this.valueChangedCallback,
      this.addImageWidget,
      this.itemCount = 3,
      this.btnPosition = UploadBtnPosition.RIGHT,
      this.shape = ImageShape.None})
      : super(key: key);

  @override
  ImageUploadPageState createState() => new ImageUploadPageState();
}

class ImageUploadPageState extends State<ImageUploadPage> {
  @override
  Widget build(BuildContext context) {
    if (widget.uploadMode == UploadMode.SINGLE) {
      String url = widget.images.isNotEmpty ? widget.images[0] : '';
      return SingleImageView(
          imageWidth: widget.imageWidth,
          imageHeight: widget.imageHeight,
          shape: widget.shape,
          imageUrl: url,
          singleCallback: (value) {
            if (widget.valueChangedCallback != null) {
              widget.valueChangedCallback!([value]);
            }
          });
    } else {
      return MultipleImageView(
        images: widget.images,
        imageWidth: widget.imageWidth,
        imageHeight: widget.imageHeight,
        addImageWidget: widget.addImageWidget,
        itemCount: widget.itemCount,
        btnPosition: widget.btnPosition,
        shape: widget.shape,
        valueChangedCallback: widget.valueChangedCallback,
      );
    }
  }

  @override
  void initState() {
    super.initState();
  }
}
