import 'dart:convert';
import 'dart:core';
import 'dart:io';
import 'package:dio/dio.dart';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:vone_ipfs/bean/base_entity.dart';
import 'package:vone_ipfs/bean/upload_file_entity.dart';
import 'package:vone_ipfs/bean/upload_result_entity.dart';
import 'package:vone_ipfs/generated/json/base/json_convert_content.dart';
import 'package:vone_ipfs/generated/json/upload_file_entity_helper.dart';
import 'package:vone_ipfs/net/http_api.dart';
import 'package:vone_ipfs/pages/image_upload_page.dart';
import 'package:vone_ipfs/utils/toast_utils.dart';
import 'package:wechat_assets_picker/wechat_assets_picker.dart';

class IPFSUtils {
  ///图片音视频上传
  static Future<void> fileUploadSingle(
      context, ValueChanged<UploadResultEntity> changed,
      {RequestType requestType = RequestType.common,
      CommonSortPathDelegate? sortPathDelegate,
      Function? progressCallback}) async {
    List<AssetEntity>? image = await AssetPicker.pickAssets(context,
        maxAssets: 1,
        requestType: requestType,
        sortPathDelegate: sortPathDelegate);
    if (image != null && image.length > 0) {
      File? file = await image[0].loadFile();
      if (file != null) {
        String filePath = file.path;
        if (filePath.isEmpty) {
          ToastUtils.show('选择的文件不存在,请重新选择');
          return;
        }
        voneFileUpload(filePath, changed, progressCallback: progressCallback);
        // int fileSize = await file.length();
        // if (fileSize > 40 * 1024 * 1024) {
        //   ToastUtils.show('文件大小超出限制');
        //   return;
        // }
        // String fileName = filePath.split("/").last;
        // FormData formData = FormData.fromMap({
        //   "file": await MultipartFile.fromFile(filePath, filename: fileName)
        // });
        // // try {
        //   Response response;
        //   Dio dio = Dio();
        //   response = await dio.post<String>(HttpApi.fileUpload, data: formData,
        //       onSendProgress: (count, total) {
        //     if (progressCallback != null) {
        //       progressCallback(count, total);
        //     }
        //   });
        //   UploadFileEntity fileData = JsonConvert.fromJsonAsT<UploadFileEntity>(
        //       jsonDecode(response.data.toString()));
        //   if (0 == fileData.code) {
        //     if (fileData.data != null) {
        //       UploadResultEntity entity = UploadResultEntity(fileName,
        //           fileData.data!.fileHash!, fileData.data?.cover ?? '');
        //       changed(entity);
        //     }
        //   } else {
        //     ToastUtils.show('文件上传失败');
        //   }
        // // } catch (e) {
        // //   print(e);
        // //   ToastUtils.show('文件上传失败');
        // // }
      } else {
        ToastUtils.show('选择的文件不存在,请重新选择');
      }
    }
  }

  static bool isVideo(String fileName) {
    int dot = fileName.lastIndexOf('.');
    String type = fileName.substring(dot + 1);
    return type.toLowerCase() == "mp4";
  }

  ///文件上传
  static Future<void> fileUpload(
      context, ValueChanged<UploadResultEntity> changed,
      {Function? progressCallback}) async {
    List<String> allowedExtensions = [
      'jpg',
      'png',
      'doc',
      'docx',
      'xls',
      'xlsx',
      'ppt',
      'pptx',
      'pdf'
    ];
    FilePickerResult? result = await FilePicker.platform
        .pickFiles(type: FileType.custom, allowedExtensions: allowedExtensions);
    if (result != null &&
        result.files.first != null &&
        result.files.first.extension != null &&
        result.files.first.extension!.isNotEmpty) {
      PlatformFile file = result.files.first;
      if (file != null) {
        String filePath = file.path ?? '';
        if (filePath.isEmpty) {
          ToastUtils.show('选择的文件不存在,请重新选择');
          return;
        }
        // int fileSize = await file.length();
        // if (fileSize > 40 * 1024 * 1024) {
        //   ToastUtils.show('文件大小超出限制');
        //   return;
        // }

        voneFileUpload(filePath, changed, progressCallback: progressCallback);
      } else {
        ToastUtils.show('选择的文件不存在,请重新选择');
      }
    }
  }

  static Future<void> voneFileUpload(
      String filePath, ValueChanged<UploadResultEntity> changed,
      {Function? progressCallback}) async {
    try {
      String fileName = filePath.split("/").last;
      FormData formData = FormData.fromMap(
          {"file": await MultipartFile.fromFile(filePath, filename: fileName)});
      Response response;
      Dio dio = Dio();
      response = await dio.post<String>("${HttpApi.fileUpload}", data: formData,
          onSendProgress: (count, total) {
        if (progressCallback != null) {
          progressCallback(count, total);
        }
      });
      UploadFileEntity fileData = JsonConvert.fromJsonAsT<UploadFileEntity>(
          jsonDecode(response.data.toString()));
      if (0 == fileData.code) {
        if (fileData.data != null) {
          UploadResultEntity entity = UploadResultEntity(
              fileName, fileData.data!.fileHash!, fileData.data?.cover ?? '');
          changed(entity);
        }
      } else {
        ToastUtils.show('文件上传失败');
      }
    } catch (e) {
      ToastUtils.show('文件上传失败');
    }
  }

  static void imageUpload(BuildContext context, List<String> list,
      {double width = 110, double height = 110}) {
    Navigator.push(context,
        MaterialPageRoute(builder: (context) => ImageUploadPage(images: list)));
  }

  ///根据文件的后缀名获取文件的图标
  static Icon getFileIconBySuffix(String path, bool isFold) {
    int iconName = 0xe858;
    Color color = const Color(0xFFFFD23D);
    int dotIndex = 0;
    String suffix = "";
    if (!isFold) {
      dotIndex = path.lastIndexOf('.');
      suffix = path.substring(dotIndex + 1).toLowerCase();
    }

    if (isFold) {
      iconName = 0xe622;
      color = const Color(0xFF4D97FF);
    } else {
      if (suffix == 'doc' || suffix == 'docx') {
        iconName = 0xe885;
        color = const Color(0xFF4D97FF);
      } else if (suffix == 'xls' || suffix == 'xlsx') {
        iconName = 0xedde;
        color = const Color(0xFF63C421);
      } else if (suffix == 'ppt' || suffix == 'pptx') {
        iconName = 0xe859;
        color = const Color(0xFFFF9743);
      } else if (suffix == 'mp3' ||
          suffix == 'wav' ||
          suffix == 'mid' ||
          suffix == 'wma' ||
          suffix == 'acc') {
        iconName = 0xe6ed;
        color = const Color(0xFF8183F1);
      } else if (suffix == 'mp4' ||
          suffix == 'avi' ||
          suffix == 'mov' ||
          suffix == 'asf' ||
          suffix == 'wmv' ||
          suffix == 'mod' ||
          suffix == 'mpeg' ||
          suffix == 'swf' ||
          suffix == 'flv' ||
          suffix == 'ogg') {
        iconName = 0xe665;
        color = const Color(0xFF8183F1);
      } else if (suffix == 'pdf') {
        iconName = 0xe85b;
        color = const Color(0xFFFF5B5A);
      } else if (suffix == 'zip' || suffix == 'rar') {
        iconName = 0xe85a;
        color = const Color(0xFF54C7F7);
      } else if (suffix == 'png' ||
          suffix == 'jpg' ||
          suffix == 'jpeg' ||
          suffix == 'bmp' ||
          suffix == 'gif') {
        iconName = 0xe858;
        color = const Color(0xFFFF7743);
      } else if (suffix == 'txt') {
        iconName = 0xe62e;
        color = const Color(0xFF4D97FF);
      }
    }
    return Icon(
      IconData(iconName, fontFamily: 'iconfont'),
      color: color,
      size: 40,
    );
  }
}

// class CustomSortPathDelegate extends CommonSortPathDelegate {
//   const CustomSortPathDelegate();
//
//   @override
//   void sort(List<AssetPathEntity> list) {
//     ///...///
//
//     // In here you can check every path entities if you want.
//     // The only property we recommend to change is [name],
//     // And we have no responsibility for issues caused by
//     // other properties update.
//     for (final AssetPathEntity entity in list) {
//       // If the entity `isAll`, that's the "Recent" entity you want.
//       if (entity.isAll) {
//         entity.name = '全部';
//       } else if (entity.name == 'Camera') {
//         entity.name = '相机';
//       } else if (entity.name == 'Pictures') {
//         entity.name = '图片';
//       }
//     }
//   }
// }

///多文件还是单文件
enum UploadMode {
  ///单文件
  SINGLE,

  ///多文件
  MULTIPLE
}

///上传图片的按钮位置
enum UploadBtnPosition {
  ///图片上面
  TOP,

  ///图片右边
  RIGHT
}
