import 'package:flutter/material.dart';
import 'package:vone_ipfs/net/http_api.dart';
import 'package:vone_ipfs/pages/image_picker_dialog.dart';
import 'package:vone_ipfs/pages/upload_progress_dialog.dart';
import 'package:vone_ipfs/utils/ipfs_utils.dart';
import 'package:vone_ipfs/widget/vone_image.dart';

import 'defult_upload_view.dart';
import 'intrinsic_grid_view.dart';

class MultipleImageView extends StatefulWidget {
  ///图片的宽度
  final double imageWidth;

  ///图片高度
  final double imageHeight;

  ///已有的图片列表
  final List<String> images;

  ///图片形状
  final ImageShape shape;

  ///上传成功的回调
  final ValueChanged<List<String>>? valueChangedCallback;

  ///添加图片的按钮
  final Widget? addImageWidget;

  /// 添加图片按钮的位置
  final UploadBtnPosition btnPosition;
  final int itemCount;

  MultipleImageView(
      {Key? key,
      this.imageWidth = 110,
      this.imageHeight = 110,
      required this.images,
      this.valueChangedCallback,
      this.addImageWidget,
      this.itemCount = 3,
      this.btnPosition = UploadBtnPosition.RIGHT,
      this.shape = ImageShape.None})
      : super(key: key);

  @override
  MultipleImageViewState createState() => new MultipleImageViewState();
}

class MultipleImageViewState extends State<MultipleImageView> {
  List<String> _imageList = [];
  bool _showProgress = false;
  int _count = 0;
  int _total = 0;

  _dismissDialog() {
    Navigator.of(context).pop();
    setState(() {
      _showProgress = false;
    });
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: const EdgeInsets.only(left: 10),
      child: Column(
        children: [
          widget.btnPosition == UploadBtnPosition.TOP
              ? _buildTopBtn()
              : const Text(''),
          IntrinsicGridView.vertical(
            children: [
              for (int i = 0; i < _imageList.length + 1; i++) _buildItem(i)
            ],
            columnCount: widget.itemCount,
            horizontalSpace: 5,
            verticalSpace: 5,
          )
        ],
      ),
    );
  }

  Widget _buildItem(int index) {
    if (_imageList.isEmpty || index == _imageList.length) {
      return widget.btnPosition == UploadBtnPosition.TOP
          ? const Text('')
          : InkWell(
              child: widget.addImageWidget ??
                  DefaultUploadView(
                    width: widget.imageWidth,
                    height: widget.imageHeight,
                  ),
              onTap: () {
                _showPickerDialog();
              },
            );
    } else {
      return Container(
        width: widget.imageWidth,
        height: widget.imageHeight,
        alignment: Alignment.topLeft,
        child: Stack(
          alignment: Alignment.topRight,
          children: [
            VoneImage(
              imageWidth: double.infinity,
              imageHeight: double.infinity,
              imageUrl: _imageList[index],
              shape: widget.shape,
              boxFit: BoxFit.fill,
            ),
            InkWell(
              child: Container(
                width: 20,
                height: 20,
                decoration: BoxDecoration(
                    color: Colors.black.withOpacity(0.6),
                    borderRadius: BorderRadius.circular(14)),
                child: const Icon(
                  Icons.close,
                  size: 20,
                  color: Colors.white,
                ),
              ),
              onTap: () {
                setState(() {
                  _imageList.removeAt(index);
                });
                if (widget.valueChangedCallback != null) {
                  widget.valueChangedCallback!(_imageList);
                }
              },
            )
          ],
        ),
      );
    }
  }

  @override
  void initState() {
    super.initState();
    _imageList = widget.images;
  }

  _showPickerDialog() {
    showModalBottomSheet(
        backgroundColor: Colors.transparent,
        context: context, //state.context,
        builder: (BuildContext context) {
          return ImagePickerDialog(
            localCallback: () {
              IPFSUtils.fileUploadSingle(context, (value) {
                String url = '${HttpApi.getFilePath}/${true}/${value.fileHash}';
                _imageList.add(url);
                setState(() {});
                if (widget.valueChangedCallback != null) {
                  widget.valueChangedCallback!(_imageList);
                }
                _dismissDialog();
              }, progressCallback: (count, total) {
                if (!_showProgress) {
                  _showUploadDialog();
                  setState(() {
                    _showProgress = true;
                  });
                }
                setState(() {
                  _count = count;
                  _total = total;
                });
              });
            },
            cloudCallback: () {
              _showUploadDialog();
            },
          );
        });
  }

  _showUploadDialog() {
    if (_showProgress) {
      return;
    }
    print("------ $_count");
    showGeneralDialog(
        context: context,
        barrierLabel: "",
        barrierColor: Colors.black.withOpacity(0.5),
        transitionDuration: const Duration(milliseconds: 500),
        barrierDismissible: true,
        pageBuilder: (BuildContext context, Animation animation,
            Animation secondaryAnimation) {
          return Scaffold(
            backgroundColor: Colors.transparent,
            body: Center(
              child: UploadProgressDialog(
                count: _count,
                total: _total,
              ),
            ),
          );
        });
  }

  Widget _buildTopBtn() {
    return InkWell(
      child: Container(
        alignment: Alignment.center,
        width: double.infinity,
        padding: const EdgeInsets.symmetric(vertical: 15),
        margin: const EdgeInsets.only(top: 10, bottom: 10, right: 10),
        decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(3), color: Colors.blue),
        child: const Text(
          '选择图片',
          style: TextStyle(color: Colors.white, fontSize: 15),
        ),
      ),
      onTap: () {
        _showPickerDialog();
      },
    );
  }
}
