import 'package:flutter/material.dart';
import 'package:vone_ipfs/net/http_api.dart';
import 'package:vone_ipfs/pages/image_picker_dialog.dart';
import 'package:vone_ipfs/utils/ipfs_utils.dart';
import 'package:vone_ipfs/widget/vone_image.dart';

import 'defult_upload_view.dart';

class SingleImageView extends StatefulWidget {
  ///图片的宽度
  final double imageWidth;

  ///图片高度
  final double imageHeight;

  ///图片形状
  final ImageShape shape;

  ///添加图片的按钮
  final Widget? addImageWidget;
  final String imageUrl;
  ValueChanged<String>? singleCallback;

  SingleImageView(
      {Key? key,
      this.imageWidth = 110,
      this.imageHeight = 110,
      this.shape = ImageShape.None,
      this.addImageWidget,
      this.singleCallback,
      this.imageUrl = ""})
      : super(key: key);

  @override
  SingleImageViewState createState() => new SingleImageViewState();
}

class SingleImageViewState extends State<SingleImageView> {
  String _imageUrl = '';

  @override
  Widget build(BuildContext context) {
    return InkWell(
      child: Container(
        width: widget.imageWidth,
        height: widget.imageHeight,
        child: _imageUrl.isEmpty
            ? (widget.addImageWidget ??
                DefaultUploadView(
                  width: widget.imageWidth,
                  height: widget.imageHeight,
                ))
            : VoneImage(
                imageWidth: double.infinity,
                imageHeight: double.infinity,
                imageUrl: _imageUrl,
                shape: widget.shape,
                boxFit: BoxFit.fill,
              ),
      ),
      onTap: () {
        _showPickerDialog();
      },
    );
  }

  @override
  void initState() {
    super.initState();
    _imageUrl = widget.imageUrl;
  }

  @override
  void dispose() {
    super.dispose();
  }

  _showPickerDialog() {
    showModalBottomSheet(
        backgroundColor: Colors.transparent,
        context: context, //state.context,
        builder: (BuildContext context) {
          return ImagePickerDialog(
            localCallback: () {
              IPFSUtils.fileUploadSingle(context, (value) {
                String url = '${HttpApi.getFilePath}/${true}/${value.fileHash}';
                _imageUrl = url;
                setState(() {});
                if (widget.singleCallback != null) {
                  widget.singleCallback!(url);
                }
              }, progressCallback: (count, total) {
                // if (!_showProgress) {
                //   _showUploadDialog();
                //   setState(() {
                //     _showProgress = true;
                //   });
                // }
                // setState(() {
                //   _count = count;
                //   _total = total;
                // });
              });
            },
            cloudCallback: () {
              // _showUploadDialog();
            },
          );
        });
  }
}
